<?php

namespace Drupal\restfullogger\Plugin\rest\resource;

use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Psr\Log\LoggerInterface;
use Psr\Log\LogLevel;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides a resource to post watchdog database logs.
 *
 * @RestResource(
 *   id = "restfullogger",
 *   label = @Translation("Watchdog database logger"),
 *   uri_paths = {
 *     "create" = "/dblog/logger"
 *   }
 * )
 */
class RestfulLogger extends ResourceBase {

    /**
     * A current user instance.
     *
     * @var \Drupal\Core\Session\AccountProxyInterface
     */
    protected $currentUser;

    /**
     * Constructs a new DefaultRestResource object.
     *
     * @param array $configuration
     *   A configuration array containing information about the plugin instance.
     * @param string $plugin_id
     *   The plugin_id for the plugin instance.
     * @param mixed $plugin_definition
     *   The plugin implementation definition.
     * @param array $serializer_formats
     *   The available serialization formats.
     * @param \Psr\Log\LoggerInterface $logger
     *   A logger instance.
     * @param \Drupal\Core\Session\AccountProxyInterface $current_user
     *   A current user instance.
     */
    public function __construct( array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, AccountProxyInterface $current_user) {
        parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
        $this->currentUser = $current_user;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
        return new static(
          $configuration,
          $plugin_id,
          $plugin_definition,
          $container->getParameter('serializer.formats'),
          $container->get('logger.factory')->get('restfullogger'),
          $container->get('current_user')
        );
    }

  /**
   * Responds to POST requests.
   *
   * @param array $request
   *   The current request.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   */
    public function post(array $request) {
        // Use current user after pass authentication to validate access.
        if (!$this->currentUser->hasPermission('post log messages')) {
            throw new AccessDeniedHttpException();
        }
        // Validate that a message is there.
        $message = $this->validateAndParseMessage($request);
        $channel = empty($request['channel']) ? 'restfullogger' : $request['channel'];
        $severity = $this->validateAndParseSeverity($request);
        $path = $this->validateAndParsePath($request);
        $message = t('@message, path:  @path', [
          '@message' => $message,
          '@path' => $path,
        ]);

        // Log the message.
        $this->logger->log($severity, $message);

        // Build our response.
        $response = [
          'message' => 'successfully logged',
          'dblog_message' => $message,
          'dblog_channel' => $channel,
          'dblog_severity' => $severity,
        ];

        return new ModifiedResourceResponse($response, 200);
    }

  /**
   * Validates and extracts the message to log.
   *
   * @param array $request
   *
   * @throws \Symfony\Component\HttpKernel\Exception\BadRequestHttpException
   *   Thrown when the request is mal-formatted.
   *
   * @return resource|string
   */
  private function validateAndParseMessage(array $request) {
    // Check if the message value exists.
    if (empty($request['message'])) {
      throw new BadRequestHttpException(t('The message value is missing'));
    }
    return $request['message'];
  }

  /**
   * Validates and extracts the severity of the message.
   *
   * @param array $request
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup|string
   */
  private function validateAndParseSeverity(array $request) {
    $levels = RfcLogLevel::getLevels();
    $severity = empty($request['severity']) ? LogLevel::NOTICE : $this->t('@severity', [
      '@severity' => strtolower($request['severity']),
    ]);
    // Default 'Notice' if the level provided is not available.
    return in_array(ucfirst($severity), $levels) ? $severity : LogLevel::NOTICE;
  }

  /**
   * Validates and extracts the path from the request.
   *
   * @param array $request
   *
   * @throws \Symfony\Component\HttpKernel\Exception\BadRequestHttpException
   *   Thrown when the request is mal-formatted.
   *
   * @return resource|string
   */
  private function validateAndParsePath(array $request) {
    // Check if the path value exists.
    if (empty($request['path'])) {
      throw new BadRequestHttpException(t('The path value is missing'));
    }
    return $request['path'];
  }

}
